/*

*/

import java.math.BigInteger;
import java.util.List;
import java.util.ArrayList;

/**
   This program takes a single command-line argument which is an integer.

   This program creates a list of the integer's prime factors
   and then prints that list to standard output. The first item
   in the output list is the given integer to be factored and
   the last item in the output list is the number of milliseconds
   that it took to so the factorization.

   Try  37591057185623057848 (approximately 30 seconds)
        37591057185623757848 (approximately 80 seconds)
*/
public class FactorIntoPrimes
{
   public static void main(String[] args)
   {
      if (args.length == 0)
      {
         System.err.println("Usage: java FactorPrimes <big integer>");
         System.exit(0);
      }

      BigInteger i = new BigInteger(args[0]);

      System.out.println(factorize(i));
   }


   /**
      A naive factorization method. Take integer n, return list of factors.
   */
   public static List<BigInteger> factorize(BigInteger n)
   {
      List<BigInteger> factors = new ArrayList<>();

      factors.add(n);

      final long startTime = System.currentTimeMillis();

      if (n.compareTo(BigInteger.valueOf(2)) < 0)
         return factors;

      BigInteger p = BigInteger.valueOf(2);

      boolean done = false;
      while (! done)
      {
         if ( n.equals(BigInteger.ONE) )
            done = true;

         BigInteger r = n.remainder(p);
         if ( r.equals(BigInteger.ZERO) )
         {
            factors.add(p);
            n = n.divide(p);
         }
         else if (p.pow(2).compareTo(n) >= 0)
         {
            factors.add(n);
            done = true;
         }
         else if (p.compareTo(BigInteger.valueOf(2)) > 0)
         {
            p = p.add(BigInteger.valueOf(2)); // advance in steps of 2 over odd numbers
         }
         else
         {
            p = BigInteger.valueOf(3); // if p == 2, get to 3
         }
      }
      factors.add(BigInteger.ZERO);
      final long stopTime = System.currentTimeMillis();
      factors.add(BigInteger.valueOf(stopTime - startTime));
      return factors;
   }
}
