/*
   This program demonstrates the Java API for creating a process.

   Make sure that you compile FactorIntoPrimes.java before running this program.

   See
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/lang/ProcessBuilder.html
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/lang/Process.html
*/

import java.io.File;

/**
   This program uses the Java Process API to run
   FactorIntoPrimes.java on a couple of integers.

   The results that FactorIntoPrimes.java writes
   to its standard output are redirected to a file
   in the current directory.
*/
public class RunProcess
{
   public static void main(String[] args)
   {
      final String number1 = "37591057185623057848";
      final String number2 = "375910571856230578480";

      final long startTime = System.currentTimeMillis();
      try
      {
         // Create a command-line for running the program.
         final ProcessBuilder pb1 = new ProcessBuilder(
                                         "java.exe",
                                         "FactorIntoPrimes",
                                         number1);
         final File out1 = new File(number1 + ".txt");
         pb1.redirectOutput(ProcessBuilder.Redirect.appendTo(out1));
         // Execute the command-line.
         final Process p1 = pb1.start();

         // Create a command-line for running the program.
         final ProcessBuilder pb2 = new ProcessBuilder(
                                         "java.exe",
                                         "FactorIntoPrimes",
                                         number2);
         final File out2 = new File(number2 + ".txt");
         pb2.redirectOutput(ProcessBuilder.Redirect.appendTo(out2));
         // Execute the command-line.
         final Process p2 = pb2.start();

         p1.waitFor();
         p2.waitFor();
      }
      catch (Exception e)
      {
         System.out.println( e );
      }
      final long stopTime = System.currentTimeMillis();
      System.out.println( (stopTime - startTime) + " ms" );
   }
}
